#! /usr/bin/python3
# -*- coding: utf-8 -*-

# Copyright (C) 2013 name <email>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from gi.repository import Unity, UnityExtras
from gi.repository import Gio, GLib
import urllib.parse
import urllib.request
import json
import datetime
import gettext

APP_NAME = 'unity-scope-openweathermap'
LOCAL_PATH = '/usr/share/locale/'
gettext.bindtextdomain(APP_NAME, LOCAL_PATH)
gettext.textdomain(APP_NAME)
_ = gettext.gettext

GROUP_NAME = 'com.canonical.Unity.Scope.Info.Openweathermap'
UNIQUE_PATH = '/com/canonical/unity/scope/info/openweathermap'
SEARCH_URI = ['http://openweathermap.org/data/2.1/find/', 
              'http://openweathermap.org/data/2.1/forecast/city/']
SEARCH_HINT = _('Search OpenWeatherMap')
NO_RESULTS_HINT = _('Sorry, there is no OpenwWeatherMap forecast that matches your search.')
PROVIDER_CREDITS = _('Powered by OpenWeatherMap')
SVG_DIR = '/usr/share/icons/unity-icon-theme/places/svg/'
PROVIDER_ICON = SVG_DIR+'service-openweathermap.svg'
DEFAULT_RESULT_ICON = SVG_DIR+'result-info.svg'
DEFAULT_RESULT_MIMETYPE = 'text/html'
DEFAULT_RESULT_TYPE = Unity.ResultType.DEFAULT

c1 = {'id'      :'Cat 1',
      'name'    :_('Weather Forecast'),
      'icon'    :SVG_DIR+'group-info.svg',
      'renderer':Unity.CategoryRenderer.VERTICAL_TILE}
CATEGORIES = [c1]
FILTERS = []
EXTRA_METADATA = []

def weather_search(query, s_type):
    print (query)
    query = urllib.parse.quote(str(query))
    data = None
    if not query:
        return data
    if s_type == 'forecast':
        uri = "%s%s?mode=daily_compact&units=metric" % (SEARCH_URI[1], query)
    else:
        uri = "%sname?units=metric&type=like&q=%s" % (SEARCH_URI[0], query)
    print (uri)
    try:
        response = urllib.request.urlopen(uri).read()
        data = json.loads(response.decode('utf8'))
    except Exception as error:
        print (error)
    return data

def search(search, filters):
    '''
    Any search method returning results as a list of tuples.
    Available tuple fields:
    uri (string)
    icon (string)
    title (string)
    comment (string)
    dnd_uri (string)
    mimetype (string)
    category (int)
    result_type (Unity ResultType)
    extras metadata fields (variant)
    '''
    results = []
    place_data = weather_search(search, 'place')
    if not place_data or not "list" in place_data:
        return results
    uri, city_id, title, wind, icon, description = None, None, None, None, None, None
    try:
        description = place_data['list'][0]['weather'][0]['description']
        icon = "http://openweathermap.org/img/w/" + place_data['list'][0]['weather'][0]['icon'] + ".png"
        wind =  place_data['list'][0]['wind']['speed']
        temp_c = float(place_data['list'][0]['main']['temp'])
        temp_f = temp_c*(9.0/5.0)+32
        temp = '%i%sC/%i%sF' % (int(temp_c),  u"\u00B0", int(temp_f),  u"\u00B0")
        title = _('Today') + '\n' + temp
        city_id = place_data['list'][0]['id']
        uri = place_data['list'][0]['url']
    except Exception as error:
        print(error)
        return results
    if not uri or not city_id:
        return results
    results.append({'uri':uri,
                    'icon':icon,
                    'title':title,
                    'comment':description})
    forecast = weather_search(city_id, 'forecast')
    if not forecast or not "list" in forecast:
        return results
    today = datetime.datetime.now().today()
    for i in range(0,4):
        if i == 0:
            day_name = _('Tomorrow')
        else:
            day = today + datetime.timedelta(days=i+1)
            day_name = day.strftime("%A")
        title, wind, icon, description = None, None, None, None
        try:
            description = forecast['list'][i]['weather'][0]['description']
            icon = "http://openweathermap.org/img/w/" + forecast['list'][i]['weather'][0]['icon'] + ".png"
            wind =  forecast['list'][i]['speed']
            temp_c = float(forecast['list'][i]['temp'])
            temp_f = temp_c*(9.0/5.0)+32
            temp = '%i%sC/%i%sF' % (int(temp_c),  u"\u00B0", int(temp_f),  u"\u00B0")
            title = day_name + '\n' + temp
        except Exception as error:
            print(error)
            return results
        results.append({'uri':uri,
                        'icon':icon,
                        'title':title,
                        'comment':description})
    return results


# Classes below this point establish communication
# with Unity, you probably shouldn't modify them.


class MySearch (Unity.ScopeSearchBase):
    def __init__(self, search_context):
        super (MySearch, self).__init__()
        self.set_search_context (search_context)

    def do_run (self):
        '''
        Adds results to the model
        '''
        try:
            result_set = self.search_context.result_set
            for i in search(self.search_context.search_query,
                            self.search_context.filter_state):
                if not 'uri' in i or not i['uri'] or i['uri'] == '':
                    continue
                if not 'icon' in i or not i['icon'] or i['icon'] == '':
                    i['icon'] = DEFAULT_RESULT_ICON
                if not 'mimetype' in i or not i['mimetype'] or i['mimetype'] == '':
                    i['mimetype'] = DEFAULT_RESULT_MIMETYPE
                if not 'result_type' in i or not i['result_type'] or i['result_type'] == '':
                    i['result_type'] = DEFAULT_RESULT_TYPE
                if not 'category' in i or not i['category'] or i['category'] == '':
                    i['category'] = 0
                if not 'title' in i or not i['title']:
                    i['title'] = ''
                if not 'comment' in i or not i['comment']:
                    i['comment'] = ''
                if not 'dnd_uri' in i or not i['dnd_uri'] or i['dnd_uri'] == '':
                    i['dnd_uri'] = i['uri']
                i['metadata'] = {}
                if EXTRA_METADATA:
                    for e in i:
                        for m in EXTRA_METADATA:
                            if m['id'] == e:
                                i['metadata'][e] = i[e]
                i['metadata']['provider_credits'] = GLib.Variant('s', PROVIDER_CREDITS)
                result = Unity.ScopeResult.create(str(i['uri']), str(i['icon']),
                                                  i['category'], i['result_type'],
                                                  str(i['mimetype']), str(i['title']),
                                                  str(i['comment']), str(i['dnd_uri']),
                                                  i['metadata'])
                result_set.add_result(result)
        except Exception as error:
            print (error)

class Scope (Unity.AbstractScope):
    def __init__(self):
        Unity.AbstractScope.__init__(self)

    def do_get_search_hint (self):
        return SEARCH_HINT

    def do_get_schema (self):
        '''
        Adds specific metadata fields
        '''
        schema = Unity.Schema.new ()
        if EXTRA_METADATA:
            for m in EXTRA_METADATA:
                schema.add_field(m['id'], m['type'], m['field'])
        #FIXME should be REQUIRED for credits
        schema.add_field('provider_credits', 's', Unity.SchemaFieldType.OPTIONAL)
        return schema

    def do_get_categories (self):
        '''
        Adds categories
        '''
        cs = Unity.CategorySet.new ()
        if CATEGORIES:
            for c in CATEGORIES:
                cat = Unity.Category.new (c['id'], c['name'],
                                          Gio.ThemedIcon.new(c['icon']),
                                          c['renderer'])
                cs.add (cat)
        return cs

    def do_get_filters (self):
        '''
        Adds filters
        '''
        fs = Unity.FilterSet.new ()
#        if FILTERS:
#            
        return fs

    def do_get_group_name (self):
        return GROUP_NAME

    def do_get_unique_name (self):
        return UNIQUE_PATH

    def do_create_search_for_query (self, search_context):
        se = MySearch (search_context)
        return se

def load_scope():
    return Scope()
